@@ -153,6 +153,7 @@ public:
 
 private:
 
+    /// @public
     /// @brief Assign class using vendor-class-identifier option
     ///
     /// @note This is the first part of @ref classifyPacket
@@ -330,8 +331,7 @@ public:
     /// methods, generates appropriate answer, sends the answer to the client.
     ///
     /// @param query A pointer to the packet to be processed.
-    /// @param rsp A pointer to the response
-    void processPacketAndSendResponse(Pkt4Ptr& query, Pkt4Ptr& rsp);
+    void processPacketAndSendResponse(Pkt4Ptr& query);
 
     /// @brief Process a single incoming DHCPv4 packet and sends the response.
     ///
@@ -339,8 +339,7 @@ public:
     /// methods, generates appropriate answer, sends the answer to the client.
     ///
     /// @param query A pointer to the packet to be processed.
-    /// @param rsp A pointer to the response
-    void processPacketAndSendResponseNoThrow(Pkt4Ptr& query, Pkt4Ptr& rsp);
+    void processPacketAndSendResponseNoThrow(Pkt4Ptr& query);
 
     /// @brief Process an unparked DHCPv4 packet and sends the response.
     ///
@@ -1029,6 +1028,7 @@ protected:
 
 private:
 
+    /// @public
     /// @brief Assign class using vendor-class-identifier option
     ///
     /// @note This is the first part of @ref classifyPacket
