@@ -227,11 +227,14 @@ ControlledDhcpv6Srv::commandShutdownHandler(const string&, ConstElementPtr args)
     }
 
     ControlledDhcpv6Srv::getInstance()->shutdownServer(exit_value);
-    return(createAnswer(CONTROL_RESULT_SUCCESS, "Shutting down."));
+    return (createAnswer(CONTROL_RESULT_SUCCESS, "Shutting down."));
 }
 
 ConstElementPtr
 ControlledDhcpv6Srv::commandLibReloadHandler(const string&, ConstElementPtr) {
+    // stop thread pool (if running)
+    MultiThreadingCriticalSection cs;
+
     /// @todo delete any stored CalloutHandles referring to the old libraries
     /// Get list of currently loaded libraries and reload them.
     HookLibsCollection loaded = HooksManager::getLibraryInfo();
@@ -355,6 +358,14 @@ ControlledDhcpv6Srv::commandConfigSetHandler(const string&,
         return (result);
     }
 
+    // stop thread pool (if running)
+    MultiThreadingCriticalSection cs;
+
+    // disable multi-threading (it will be applied by new configuration)
+    // this must be done in order to properly handle MT to ST transition
+    // when 'multi-threading' structure is missing from new config
+    MultiThreadingMgr::instance().apply(false, 0, 0);
+
     // We are starting the configuration process so we should remove any
     // staging configuration that has been created during previous
     // configuration attempts.
@@ -444,11 +455,6 @@ ControlledDhcpv6Srv::commandConfigTestHandler(const string&,
         return (result);
     }
 
-    // We are starting the configuration process so we should remove any
-    // staging configuration that has been created during previous
-    // configuration attempts.
-    CfgMgr::instance().rollback();
-
     // Check obsolete objects.
 
     // Relocate Logging. Note this allows to check the loggers configuration.
@@ -456,6 +462,14 @@ ControlledDhcpv6Srv::commandConfigTestHandler(const string&,
 
     // Log obsolete objects and return an error.
 
+    // stop thread pool (if running)
+    MultiThreadingCriticalSection cs;
+
+    // We are starting the configuration process so we should remove any
+    // staging configuration that has been created during previous
+    // configuration attempts.
+    CfgMgr::instance().rollback();
+
     // Now we check the server proper.
     return (checkConfig(dhcp6));
 }
@@ -582,6 +596,9 @@ ControlledDhcpv6Srv::commandConfigBackendPullHandler(const std::string&,
         return (createAnswer(CONTROL_RESULT_EMPTY, "No config backend."));
     }
 
+    // stop thread pool (if running)
+    MultiThreadingCriticalSection cs;
+
     // Reschedule the periodic CB fetch.
     if (TimerMgr::instance()->isTimerRegistered("Dhcp6CBFetchTimer")) {
         TimerMgr::instance()->cancel("Dhcp6CBFetchTimer");
@@ -665,8 +682,8 @@ ControlledDhcpv6Srv::processCommand(const string& command,
 
     if (!srv) {
         ConstElementPtr no_srv = isc::config::createAnswer(1,
-          "Server object not initialized, can't process command '" +
-          command + "', arguments: '" + txt + "'.");
+            "Server object not initialized, so can't process command '" +
+            command + "', arguments: '" + txt + "'.");
         return (no_srv);
     }
 
@@ -722,7 +739,8 @@ ControlledDhcpv6Srv::processCommand(const string& command,
 
     } catch (const Exception& ex) {
         return (isc::config::createAnswer(1, "Error while processing command '"
-                                          + command + "':" + ex.what()));
+                                          + command + "':" + ex.what() +
+                                          ", params: '" + txt + "'"));
     }
 }
 
@@ -898,7 +916,10 @@ ControlledDhcpv6Srv::processConfig(isc::data::ConstElementPtr config) {
         // operation.
     }
 
-    // Configure multi threading
+    // Apply multi threading settings.
+    // @note These settings are applied/updated only if no errors occur while
+    // applying the new configuration.
+    // @todo This should be fixed.
     try {
         CfgMultiThreading::apply(CfgMgr::instance().getStagingCfg()->getDHCPMultiThreading());
         if (MultiThreadingMgr::instance().getMode()) {
@@ -1188,6 +1209,9 @@ ControlledDhcpv6Srv::dbLostCallback(ReconnectCtlPtr db_reconnect_ctl) {
 void
 ControlledDhcpv6Srv::cbFetchUpdates(const SrvConfigPtr& srv_cfg,
                                     boost::shared_ptr<unsigned> failure_count) {
+    // stop thread pool (if running)
+    MultiThreadingCriticalSection cs;
+
     try {
         // Fetch any configuration backend updates since our last fetch.
         server_->getCBControl()->databaseConfigFetch(srv_cfg,
