@@ -1,4 +1,4 @@
-// Copyright (C) 2015-2017 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2015-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -89,7 +89,14 @@ public:
         PARSER_HOOKS_LIBRARY,
 
         /// This will parse the input as dhcp-ddns. (D2 client config)
-        PARSER_DHCP_DDNS
+        PARSER_DHCP_DDNS,
+
+        /// This will parse the input as config-control.
+        PARSER_CONFIG_CONTROL,
+
+        /// This will parse the content of Logging.
+        PARSER_LOGGING
+
     } ParserType;
 
     /// @brief Default constructor.
@@ -214,10 +221,13 @@ public:
         /// Used while parsing Dhcp6/interfaces structures.
         INTERFACES_CONFIG,
 
+        /// Sanity checks.
+        SANITY_CHECKS,
+
         /// Used while parsing Dhcp6/lease-database structures.
         LEASE_DATABASE,
 
-        /// Used while parsing Dhcp6/hosts-database structures.
+        /// Used while parsing Dhcp6/hosts-database[s] structures.
         HOSTS_DATABASE,
 
         /// Used while parsing Dhcp6/*-database/type.
@@ -238,7 +248,7 @@ public:
         /// Used while parsing shared-networks structures.
         SHARED_NETWORK,
 
-        /// Used while parsing Dhcp6/Subnet6/reservation-mode.
+        /// Used while parsing Dhcp6/reservation-mode.
         RESERVATION_MODE,
 
         /// Used while parsing Dhcp6/option-def structures.
@@ -264,6 +274,12 @@ public:
         /// Used while parsing Dhcp6/control-socket structures.
         CONTROL_SOCKET,
 
+        /// Used while parsing Dhcp6/dhcp-queue-control structures.
+        DHCP_QUEUE_CONTROL,
+
+        /// Used while parsing Dhcp6/multi-threading structures.
+        DHCP_MULTI_THREADING,
+
         /// Used while parsing Dhcp6/subnet6/pools structures.
         POOLS,
 
@@ -276,13 +292,10 @@ public:
         /// Used while parsing Dhcp6/subnet6/relay structures.
         RELAY,
 
-        /// Used while parsing Dhcp6/client-classes structures.
-        CLIENT_CLASS,
-
-        /// Used while parsing Logging/loggers structures.
+        /// Used while parsing Dhcp6/loggers structures.
         LOGGERS,
 
-        /// Used while parsing Logging/loggers/output_options structures.
+        /// Used while parsing Dhcp6/loggers/output_options structures.
         OUTPUT_OPTIONS,
 
         /// Used while parsing Dhcp6/dhcp-ddns.
@@ -295,7 +308,13 @@ public:
         NCR_FORMAT,
 
         /// Used while parsing Dhcp6/dhcp-ddns/replace-client-name.
-        REPLACE_CLIENT_NAME
+        REPLACE_CLIENT_NAME,
+
+        /// Used while parsing Dhcp6/config-control
+        CONFIG_CONTROL,
+
+        /// Used while parsing config-control/config-databases
+        CONFIG_DATABASE
 
     } ParserContext;
 
