@@ -8,7 +8,9 @@
 
 #include <communication_state.h>
 #include <exceptions/exceptions.h>
+#include <ha_log.h>
 #include <ha_service_states.h>
+#include <cc/data.h>
 #include <exceptions/exceptions.h>
 #include <dhcp/dhcp4.h>
 #include <dhcp/dhcp6.h>
@@ -26,6 +28,7 @@ using namespace isc::asiolink;
 using namespace isc::data;
 using namespace isc::dhcp;
 using namespace isc::http;
+using namespace isc::log;
 using namespace boost::posix_time;
 
 namespace {
@@ -50,7 +53,8 @@ CommunicationState::CommunicationState(const IOServicePtr& io_service,
       poke_time_(boost::posix_time::microsec_clock::universal_time()),
       heartbeat_impl_(0), partner_state_(-1), partner_scopes_(),
       clock_skew_(0, 0, 0, 0), last_clock_skew_warn_(),
-      my_time_at_skew_(), partner_time_at_skew_() {
+      my_time_at_skew_(), partner_time_at_skew_(),
+      analyzed_messages_count_(0) {
 }
 
 CommunicationState::~CommunicationState() {
@@ -154,10 +158,11 @@ CommunicationState::poke() {
     // Set poke time to the current time.
     poke_time_ = boost::posix_time::microsec_clock::universal_time();
 
-    // If we have been tracking the unanswered DHCP messages directed to the
-    // partner, we need to clear any gathered information because the connection
+    // If we have been tracking the DHCP messages directed to the partner,
+    // we need to clear any gathered information because the connection
     // seems to be (re)established.
-    clearUnackedClients();
+    clearConnectingClients();
+    analyzed_messages_count_ = 0;
 
     if (timer_) {
         // Check the duration since last poke. If it is less than a second, we don't
@@ -188,6 +193,11 @@ CommunicationState::isCommunicationInterrupted() const {
     return (getDurationInMillisecs() > config_->getMaxResponseDelay());
 }
 
+size_t
+CommunicationState::getAnalyzedMessagesCount() const {
+    return (analyzed_messages_count_);
+}
+
 bool
 CommunicationState::clockSkewShouldWarn() {
     // First check if the clock skew is beyond the threshold.
@@ -262,9 +272,47 @@ CommunicationState::logFormatClockSkew() const {
     return (os.str());
 }
 
+ElementPtr
+CommunicationState::getReport() const {
+    auto report = Element::createMap();
+
+    auto in_touch = (getPartnerState() > 0);
+    report->set("in-touch", Element::create(in_touch));
+
+    auto age = in_touch ? static_cast<long long int>(getDurationInMillisecs() / 1000) : 0;
+    report->set("age", Element::create(age));
+
+    try {
+        report->set("last-state", Element::create(stateToString(getPartnerState())));
+
+    } catch (...) {
+        report->set("last-state", Element::create(std::string()));
+    }
+
+    auto list = Element::createList();
+    for (auto scope : getPartnerScopes()) {
+        list->add(Element::create(scope));
+    }
+    report->set("last-scopes", list);
+    report->set("communication-interrupted",
+                Element::create(isCommunicationInterrupted()));
+    report->set("connecting-clients", Element::create(static_cast<long long>(getConnectingClientsCount())));
+    report->set("unacked-clients", Element::create(static_cast<long long>(getUnackedClientsCount())));
+
+    long long unacked_clients_left = 0;
+    if (isCommunicationInterrupted() && (config_->getMaxUnackedClients() > getUnackedClientsCount())) {
+        unacked_clients_left = static_cast<long long>(config_->getMaxUnackedClients() -
+                                                      getUnackedClientsCount());
+    }
+    report->set("unacked-clients-left", Element::create(unacked_clients_left));
+    report->set("analyzed-packets", Element::create(static_cast<long long>(getAnalyzedMessagesCount())));
+
+    return (report);
+}
+
 CommunicationState4::CommunicationState4(const IOServicePtr& io_service,
                                          const HAConfigPtr& config)
-    : CommunicationState(io_service, config), unacked_clients_() {
+    : CommunicationState(io_service, config), connecting_clients_() {
 }
 
 void
@@ -275,6 +323,8 @@ CommunicationState4::analyzeMessage(const boost::shared_ptr<dhcp::Pkt>& message)
         isc_throw(BadValue, "DHCP message to be analyzed is not a DHCPv4 message");
     }
 
+    ++analyzed_messages_count_;
+
     // Check value of the "secs" field by comparing it with the configured
     // threshold.
     uint16_t secs = msg->getSecs();
@@ -286,17 +336,11 @@ CommunicationState4::analyzeMessage(const boost::shared_ptr<dhcp::Pkt>& message)
         secs = ((secs >> 8) | (secs << 8));
     }
 
-    // Check the value of the "secs" field. If it is below the threshold there
-    // is nothing to do. The "secs" field holds a value in seconds, hence we
-    // have to multiple by 1000 to get a value in milliseconds.
-    if (secs * 1000 <= config_->getMaxAckDelay()) {
-        return;
-    }
-
-    // The "secs" value is above the threshold so we should count it as unacked
-    // request, but we will first have to check if there is such request already
-    // recorded.
-    auto existing_requests = unacked_clients_.equal_range(msg->getHWAddr()->hwaddr_);
+    // Check the value of the "secs" field. The "secs" field holds a value in
+    // seconds, hence we have to multiple by 1000 to get a value in milliseconds.
+    // If the secs value is above the threshold, it means that the current
+    // client should be considered unacked.
+    auto unacked = (secs * 1000 > config_->getMaxAckDelay());
 
     // Client identifier will be stored together with the hardware address. It
     // may remain empty if the client hasn't specified it.
@@ -306,34 +350,75 @@ CommunicationState4::analyzeMessage(const boost::shared_ptr<dhcp::Pkt>& message)
         client_id = opt_client_id->getData();
     }
 
-    // Iterate over the requests we found so far and see if we have a match with
-    // the client identifier (this includes empty client identifiers).
-    for (auto r = existing_requests.first; r != existing_requests.second; ++r) {
-        if (r->second == client_id) {
-            // There is a match so we have already recorded this client as
-            // unacked.
-            return;
+    bool log_unacked = false;
+
+    // Check if the given client was already recorded.
+    auto& idx = connecting_clients_.get<0>();
+    auto existing_request = idx.find(boost::make_tuple(msg->getHWAddr()->hwaddr_, client_id));
+    if (existing_request != idx.end()) {
+        // If the client was recorded and was not considered unacked
+        // but it should be considered unacked as a result of processing
+        // this packet, let's update the recorded request to mark the
+        // client unacked.
+        if (!existing_request->unacked_ && unacked) {
+            ConnectingClient4 connecting_client{ msg->getHWAddr()->hwaddr_, client_id, unacked };
+            idx.replace(existing_request, connecting_client);
+            log_unacked = true;
+        }
+
+    } else {
+        // This is the first time we see the packet from this client. Let's
+        // record it.
+        ConnectingClient4 connecting_client{ msg->getHWAddr()->hwaddr_, client_id, unacked };
+        idx.insert(connecting_client);
+        log_unacked = unacked;
+
+        if (!unacked) {
+            // This is the first time we see this client after getting into the
+            // communication interrupted state. But, this client hasn't been
+            // yet trying log enough to be considered unacked.
+            LOG_INFO(ha_logger, HA_COMMUNICATION_INTERRUPTED_CLIENT4)
+                .arg(message->getLabel());
         }
     }
 
-    // New unacked client detected, so record the required information.
-    unacked_clients_.insert(std::make_pair(msg->getHWAddr()->hwaddr_, client_id));
+    // Only log the first time we detect a client is unacked.
+    if (log_unacked) {
+        unsigned unacked_left = 0;
+        if (config_->getMaxUnackedClients() > getUnackedClientsCount()) {
+            unacked_left = config_->getMaxUnackedClients() > getUnackedClientsCount();
+        }
+        LOG_INFO(ha_logger, HA_COMMUNICATION_INTERRUPTED_CLIENT4_UNACKED)
+            .arg(message->getLabel())
+            .arg(getUnackedClientsCount())
+            .arg(unacked_left);
+    }
 }
 
 bool
 CommunicationState4::failureDetected() const {
     return ((config_->getMaxUnackedClients() == 0) ||
-            (unacked_clients_.size() > config_->getMaxUnackedClients()));
+            (getUnackedClientsCount() > config_->getMaxUnackedClients()));
+}
+
+size_t
+CommunicationState4::getConnectingClientsCount() const {
+    return (connecting_clients_.size());
+}
+
+size_t
+CommunicationState4::getUnackedClientsCount() const {
+    return (connecting_clients_.get<1>().count(true));
 }
 
 void
-CommunicationState4::clearUnackedClients() {
-    unacked_clients_.clear();
+CommunicationState4::clearConnectingClients() {
+    connecting_clients_.clear();
 }
 
 CommunicationState6::CommunicationState6(const IOServicePtr& io_service,
                                          const HAConfigPtr& config)
-    : CommunicationState(io_service, config), unacked_clients_() {
+    : CommunicationState(io_service, config), connecting_clients_() {
 }
 
 void
@@ -344,32 +429,85 @@ CommunicationState6::analyzeMessage(const boost::shared_ptr<dhcp::Pkt>& message)
         isc_throw(BadValue, "DHCP message to be analyzed is not a DHCPv6 message");
     }
 
+    ++analyzed_messages_count_;
+
     // Check the value of the "elapsed time" option. If it is below the threshold
     // there is nothing to do. The "elapsed time" option holds the time in
     // 1/100 of second, hence we have to multiply by 10 to get a value in milliseconds.
     OptionUint16Ptr elapsed_time = boost::dynamic_pointer_cast<
         OptionUint16>(msg->getOption(D6O_ELAPSED_TIME));
-    if (!elapsed_time || elapsed_time->getValue() * 10 <= config_->getMaxAckDelay()) {
-        return;
-    }
+    auto unacked = (elapsed_time && elapsed_time->getValue() * 10 > config_->getMaxAckDelay());
 
     // Get the DUID of the client to see if it hasn't been recorded already.
     OptionPtr duid = msg->getOption(D6O_CLIENTID);
-    if (duid && unacked_clients_.count(duid->getData()) == 0) {
-        // New unacked client detected, so record the required information.
-        unacked_clients_.insert(duid->getData());
+    if (!duid) {
+        return;
+    }
+
+    bool log_unacked = false;
+
+    // Check if the given client was already recorded.
+    auto& idx = connecting_clients_.get<0>();
+    auto existing_request = idx.find(duid->getData());
+    if (existing_request != idx.end()) {
+        // If the client was recorded and was not considered unacked
+        // but it should be considered unacked as a result of processing
+        // this packet, let's update the recorded request to mark the
+        // client unacked.
+        if (!existing_request->unacked_ && unacked) {
+            ConnectingClient6 connecting_client{ duid->getData(), unacked };
+            idx.replace(existing_request, connecting_client);
+            log_unacked = true;
+        }
+
+    } else {
+        // This is the first time we see the packet from this client. Let's
+        // record it.
+        ConnectingClient6 connecting_client{ duid->getData(), unacked };
+        idx.insert(connecting_client);
+        log_unacked = unacked;
+
+        if (!unacked) {
+            // This is the first time we see this client after getting into the
+            // communication interrupted state. But, this client hasn't been
+            // yet trying log enough to be considered unacked.
+            LOG_INFO(ha_logger, HA_COMMUNICATION_INTERRUPTED_CLIENT6)
+                .arg(message->getLabel());
+        }
+    }
+
+    // Only log the first time we detect a client is unacked.
+    if (log_unacked) {
+        unsigned unacked_left = 0;
+        if (config_->getMaxUnackedClients() > getUnackedClientsCount()) {
+            unacked_left = config_->getMaxUnackedClients() > getUnackedClientsCount();
+        }
+        LOG_INFO(ha_logger, HA_COMMUNICATION_INTERRUPTED_CLIENT6_UNACKED)
+            .arg(message->getLabel())
+            .arg(getUnackedClientsCount())
+            .arg(unacked_left);
     }
 }
 
 bool
 CommunicationState6::failureDetected() const {
     return ((config_->getMaxUnackedClients() == 0) ||
-            (unacked_clients_.size() > config_->getMaxUnackedClients()));
+            (getUnackedClientsCount() > config_->getMaxUnackedClients()));
+}
+
+size_t
+CommunicationState6::getConnectingClientsCount() const {
+    return (connecting_clients_.size());
+}
+
+size_t
+CommunicationState6::getUnackedClientsCount() const {
+    return (connecting_clients_.get<1>().count(true));
 }
 
 void
-CommunicationState6::clearUnackedClients() {
-    unacked_clients_.clear();
+CommunicationState6::clearConnectingClients() {
+    connecting_clients_.clear();
 }
 
 } // end of namespace isc::ha
