@@ -318,6 +318,7 @@ IfaceMgr::addExternalSocket(int socketfd, SocketCallback callback) {
         isc_throw(BadValue, "Attempted to install callback for invalid socket "
                   << socketfd);
     }
+    std::lock_guard<std::mutex> lock(callbacks_mutex_);
     BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
         // There's such a socket description there already.
         // Update the callback and we're done
@@ -336,6 +337,12 @@ IfaceMgr::addExternalSocket(int socketfd, SocketCallback callback) {
 
 void
 IfaceMgr::deleteExternalSocket(int socketfd) {
+    std::lock_guard<std::mutex> lock(callbacks_mutex_);
+    deleteExternalSocketInternal(socketfd);
+}
+
+void
+IfaceMgr::deleteExternalSocketInternal(int socketfd) {
     for (SocketCallbackInfoContainer::iterator s = callbacks_.begin();
          s != callbacks_.end(); ++s) {
         if (s->socket_ == socketfd) {
@@ -347,6 +354,7 @@ IfaceMgr::deleteExternalSocket(int socketfd) {
 
 int
 IfaceMgr::purgeBadSockets() {
+    std::lock_guard<std::mutex> lock(callbacks_mutex_);
     std::vector<int> bad_fds;
     for (SocketCallbackInfo s : callbacks_) {
         errno = 0;
@@ -356,7 +364,7 @@ IfaceMgr::purgeBadSockets() {
     }
 
     for (auto bad_fd : bad_fds) {
-        deleteExternalSocket(bad_fd);
+        deleteExternalSocketInternal(bad_fd);
     }
 
     return (bad_fds.size());
@@ -364,6 +372,7 @@ IfaceMgr::purgeBadSockets() {
 
 void
 IfaceMgr::deleteAllExternalSockets() {
+    std::lock_guard<std::mutex> lock(callbacks_mutex_);
     callbacks_.clear();
 }
 
@@ -722,7 +731,6 @@ IfaceMgr::startDHCPReceiver(const uint16_t family) {
 
         dhcp_receiver_.reset(new WatchedThread());
         dhcp_receiver_->start(boost::bind(&IfaceMgr::receiveDHCP4Packets, this));
-
         break;
     case AF_INET6:
         // If the queue doesn't exist, packet queing has been configured
@@ -1013,9 +1021,13 @@ Pkt4Ptr IfaceMgr::receive4Indirect(uint32_t timeout_sec, uint32_t timeout_usec /
     FD_ZERO(&sockets);
 
     // if there are any callbacks for external sockets registered...
-    if (!callbacks_.empty()) {
-        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
-            addFDtoSet(s.socket_, maxfd, &sockets);
+    {
+        std::lock_guard<std::mutex> lock(callbacks_mutex_);
+        if (!callbacks_.empty()) {
+            BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+                // Add this socket to listening set
+                addFDtoSet(s.socket_, maxfd, &sockets);
+            }
         }
     }
 
@@ -1078,19 +1090,29 @@ Pkt4Ptr IfaceMgr::receive4Indirect(uint32_t timeout_sec, uint32_t timeout_usec /
         }
 
         // Let's find out which external socket has the data
-        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
-            if (!FD_ISSET(s.socket_, &sockets)) {
-                continue;
+        SocketCallbackInfo ex_sock;
+        {
+            std::lock_guard<std::mutex> lock(callbacks_mutex_);
+            BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+                if (!FD_ISSET(s.socket_, &sockets)) {
+                    continue;
+                }
+
+                // something received over external socket
+                if (s.callback_) {
+                    // Note the external socket to call its callback without
+                    // the lock taken so it can be deleted.
+                    ex_sock = s;
+                    break;
+                }
             }
+        }
 
-            // something received over external socket
-
+        if (ex_sock.callback_) {
             // Calling the external socket's callback provides its service
             // layer access without integrating any specific features
             // in IfaceMgr
-            if (s.callback_) {
-                s.callback_(s.socket_);
-            }
+            ex_sock.callback_(ex_sock.socket_);
 
             return (Pkt4Ptr());
         }
@@ -1132,10 +1154,13 @@ Pkt4Ptr IfaceMgr::receive4Direct(uint32_t timeout_sec, uint32_t timeout_usec /*
     }
 
     // if there are any callbacks for external sockets registered...
-    if (!callbacks_.empty()) {
-        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
-            // Add this socket to listening set
-            addFDtoSet(s.socket_, maxfd, &sockets);
+    {
+        std::lock_guard<std::mutex> lock(callbacks_mutex_);
+        if (!callbacks_.empty()) {
+            BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+                // Add this socket to listening set
+                addFDtoSet(s.socket_, maxfd, &sockets);
+            }
         }
     }
 
@@ -1173,19 +1198,29 @@ Pkt4Ptr IfaceMgr::receive4Direct(uint32_t timeout_sec, uint32_t timeout_usec /*
     }
 
     // Let's find out which socket has the data
-    BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
-        if (!FD_ISSET(s.socket_, &sockets)) {
-            continue;
+    SocketCallbackInfo ex_sock;
+    {
+        std::lock_guard<std::mutex> lock(callbacks_mutex_);
+        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+            if (!FD_ISSET(s.socket_, &sockets)) {
+                continue;
+            }
+
+            // something received over external socket
+            if (s.callback_) {
+                // Note the external socket to call its callback without
+                // the lock taken so it can be deleted.
+                ex_sock = s;
+                break;
+            }
         }
+    }
 
-        // something received over external socket
-
+    if (ex_sock.callback_) {
         // Calling the external socket's callback provides its service
         // layer access without integrating any specific features
         // in IfaceMgr
-        if (s.callback_) {
-            s.callback_(s.socket_);
-        }
+        ex_sock.callback_(ex_sock.socket_);
 
         return (Pkt4Ptr());
     }
@@ -1261,10 +1296,13 @@ IfaceMgr::receive6Direct(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */ )
     }
 
     // if there are any callbacks for external sockets registered...
-    if (!callbacks_.empty()) {
-        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
-            // Add it to the set as well
-            addFDtoSet(s.socket_, maxfd, &sockets);
+    {
+        std::lock_guard<std::mutex> lock(callbacks_mutex_);
+        if (!callbacks_.empty()) {
+            BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+                // Add this socket to listening set
+                addFDtoSet(s.socket_, maxfd, &sockets);
+            }
         }
     }
 
@@ -1302,19 +1340,29 @@ IfaceMgr::receive6Direct(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */ )
     }
 
     // Let's find out which socket has the data
-    BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
-        if (!FD_ISSET(s.socket_, &sockets)) {
-            continue;
+    SocketCallbackInfo ex_sock;
+    {
+        std::lock_guard<std::mutex> lock(callbacks_mutex_);
+        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+            if (!FD_ISSET(s.socket_, &sockets)) {
+                continue;
+            }
+
+            // something received over external socket
+            if (s.callback_) {
+                // Note the external socket to call its callback without
+                // the lock taken so it can be deleted.
+                ex_sock = s;
+                break;
+            }
         }
+    }
 
-        // something received over external socket
-
+    if (ex_sock.callback_) {
         // Calling the external socket's callback provides its service
         // layer access without integrating any specific features
         // in IfaceMgr
-        if (s.callback_) {
-            s.callback_(s.socket_);
-        }
+        ex_sock.callback_(ex_sock.socket_);
 
         return (Pkt6Ptr());
     }
@@ -1353,10 +1401,13 @@ IfaceMgr::receive6Indirect(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
     FD_ZERO(&sockets);
 
     // if there are any callbacks for external sockets registered...
-    if (!callbacks_.empty()) {
-        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
-            // Add it to the set as well
-            addFDtoSet(s.socket_, maxfd, &sockets);
+    {
+        std::lock_guard<std::mutex> lock(callbacks_mutex_);
+        if (!callbacks_.empty()) {
+            BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+                // Add this socket to listening set
+                addFDtoSet(s.socket_, maxfd, &sockets);
+            }
         }
     }
 
@@ -1419,19 +1470,29 @@ IfaceMgr::receive6Indirect(uint32_t timeout_sec, uint32_t timeout_usec /* = 0 */
         }
 
         // Let's find out which external socket has the data
-        BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
-            if (!FD_ISSET(s.socket_, &sockets)) {
-                continue;
+        SocketCallbackInfo ex_sock;
+        {
+            std::lock_guard<std::mutex> lock(callbacks_mutex_);
+            BOOST_FOREACH(SocketCallbackInfo s, callbacks_) {
+                if (!FD_ISSET(s.socket_, &sockets)) {
+                    continue;
+                }
+
+                // something received over external socket
+                if (s.callback_) {
+                    // Note the external socket to call its callback without
+                    // the lock taken so it can be deleted.
+                    ex_sock = s;
+                    break;
+                }
             }
+        }
 
-            // something received over external socket
-
+        if (ex_sock.callback_) {
             // Calling the external socket's callback provides its service
             // layer access without integrating any specific features
             // in IfaceMgr
-            if (s.callback_) {
-                s.callback_(s.socket_);
-            }
+            ex_sock.callback_(ex_sock.socket_);
 
             return (Pkt6Ptr());
         }
