@@ -1,4 +1,4 @@
-// Copyright (C) 2019 Internet Systems Consortium, Inc. ("ISC")
+// Copyright (C) 2019-2020 Internet Systems Consortium, Inc. ("ISC")
 //
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -181,6 +181,7 @@ TEST_F(NetworkTest, inheritanceSupport4) {
     globals_->set("ddns-qualifying-suffix", Element::create("gs"));
     globals_->set("hostname-char-set", Element::create("gc"));
     globals_->set("hostname-char-replacement", Element::create("gr"));
+    globals_->set("store-extended-info", Element::create(true));
 
     // For each parameter for which inheritance is supported run
     // the test that checks if the values are inherited properly.
@@ -317,6 +318,12 @@ TEST_F(NetworkTest, inheritanceSupport4) {
                                              &Network4::setHostnameCharReplacement,
                                              "nr", "gr");
     }
+    {
+        SCOPED_TRACE("store-extended-info");
+        testNetworkInheritance<TestNetwork4>(&Network4::getStoreExtendedInfo,
+                                             &Network4::setStoreExtendedInfo,
+                                             false, true);
+    }
 }
 
 // This test verifies that the inheritance is supported for DHCPv6
@@ -333,6 +340,7 @@ TEST_F(NetworkTest, inheritanceSupport6) {
     globals_->set("ddns-qualifying-suffix", Element::create("gs"));
     globals_->set("hostname-char-set", Element::create("gc"));
     globals_->set("hostname-char-replacement", Element::create("gr"));
+    globals_->set("store-extended-info", Element::create(true));
 
     // For each parameter for which inheritance is supported run
     // the test that checks if the values are inherited properly.
@@ -399,6 +407,12 @@ TEST_F(NetworkTest, inheritanceSupport6) {
                                              &Network6::setHostnameCharReplacement,
                                              "nr", "gr");
     }
+    {
+        SCOPED_TRACE("store-extended-info");
+        testNetworkInheritance<TestNetwork4>(&Network6::getStoreExtendedInfo,
+                                             &Network6::setStoreExtendedInfo,
+                                             false, true);
+    }
 
     // Interface-id requires special type of test.
     boost::shared_ptr<TestNetwork6> net_child(new TestNetwork6());
